"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BrowserUtilities = exports.BROKER_IN_USE_KEY = void 0;
var tslib_1 = require("tslib");
var SPKillSwitch_1 = tslib_1.__importDefault(require("./SPKillSwitch"));
var SPFlight_1 = tslib_1.__importDefault(require("./SPFlight"));
var SPExperiment_1 = require("./SPExperiment");
var _sharePointDesktopString = 'SharePointDesktop';
var _serviceWorkerProperty = 'serviceWorker';
var _navigationPreloadProperty = 'navigationPreload';
var _teamsBrowserWindowName = 'embedded-page-container';
var _teamsClientWindowName = 'extension-tab-frame';
var _vivaConnections1PAppId = 'sharepointportal_d2c6f111-ffad-42a0-b65e-ee00425598aa';
var _vivaTopicsAppId = 'sharepointportal_3ff145dc-86a3-4da8-a570-a3114e49c29a';
var _classHomeAppId = 'ClassHomePage_81fe9dd3-8207-4480-b846-cf1a967af129';
var _teamsAppEntityId = 'TeamsAppEntityId';
var _teamsHostName = 'TeamsHostName';
var _isUseTeamsAuth = 'useTeamsAuth';
/**
 * @internal
 */
exports.BROKER_IN_USE_KEY = 'useSecureBroker';
/**
 * BrowserUtilities contains common utilities for browser related operations.
 * @internal
 */
var BrowserUtilities = /** @class */ (function () {
    function BrowserUtilities() {
    }
    BrowserUtilities.isWebViewHosted = function () {
        if (BrowserUtilities._isWebViewHosted === undefined) {
            BrowserUtilities._isWebViewHosted =
                BrowserUtilities.isMobileWebView() ||
                    BrowserUtilities.isTeamsWebViewHosted() ||
                    BrowserUtilities.isTeamsAndroidHosted() ||
                    BrowserUtilities.isTeamsIOSHosted() ||
                    BrowserUtilities.isOutlookWebViewHosted() ||
                    /[?&]env=WebView/.test(location.search);
        }
        return BrowserUtilities._isWebViewHosted;
    };
    BrowserUtilities.getTeamsAppType = function () {
        var appHostType = 'SPPages';
        if (BrowserUtilities.isMEEBridgeApplicationHosted()) {
            appHostType = 'MEEBridge'; // VivaConnections Mobile
        }
        else if (BrowserUtilities.isEduClassHosted()) {
            appHostType = 'ClassHome'; // EDU
        }
        else if (BrowserUtilities.isViva1PHosted()) {
            appHostType = 'Viva1P'; // VivaConnections
        }
        else if (BrowserUtilities.isInTeamsChannelApp()) {
            appHostType = 'Channel'; // Sharepoint tab in Teams
        }
        else if (BrowserUtilities.isInPortalsApp()) {
            appHostType = 'Portals'; // LOB
        }
        else if (BrowserUtilities.isTeamsHosted()) {
            appHostType = 'Custom'; // Custom
        }
        if (BrowserUtilities.isTeamsHosted()) {
            var teamsHostType = BrowserUtilities.isTeamsBrowserHosted() ? 'TeamsWeb' : 'TeamsNative';
            var isSB = this.isUsingSecureBroker();
            appHostType = "".concat(appHostType).concat(isSB ? '_SB_' : '_').concat(teamsHostType);
            var teamsVersion = BrowserUtilities.safelyAccessLocalStorage(_teamsHostName);
            if (teamsVersion === 'TeamsModern') {
                appHostType += "_".concat(teamsVersion);
            }
        }
        return appHostType;
    };
    /** Returns the Product information for ASHA for Customer Promise implementations to consume.
     * We have SPO Sites and Connections (Viva) as 2 main identified products for many of our Customer Promise scenarios in WEX.
     * This method checks if a page is hosted within Teams and the hosted app is Viva1P to identify 'Connections',
     * else we return the default 'SPO Sites' as Product.
     * Edu scenario is categorized under SPO Sites product and will be further differentiated using endpoint value further.
     */
    BrowserUtilities.getASHAProduct = function () {
        if (BrowserUtilities.isViva1PHosted()) {
            return 'Connections';
        }
        else {
            return 'SPO Sites';
        }
    };
    /**
     * Returns true if current page has flag for using teams authentication
     * @returns
     */
    BrowserUtilities.isUseTeamsAuth = function () {
        try {
            return (BrowserUtilities.isTeamsHosted() && !!new URL(window.location.href).searchParams.get(_isUseTeamsAuth));
        }
        catch (_a) {
            // no-op, we want to default to not using teams auth if it's not set
            // (window.location.href is not a valid URL would hit here).
            return false;
        }
    };
    /**
     * Returns true if current page is loaded as iFrame under secure broker app
     * App is responsible for calling setUsingSecureBroker() before isUsingSecureBroker if value is true.
     * @returns
     */
    BrowserUtilities.isUsingSecureBroker = function () {
        var _a, _b;
        try {
            if (!SPKillSwitch_1.default.isActivated('6efb55f0-7812-4b62-9443-988f9d8bbdbe'
            /* 02/24/2023, 'Any teams hosted scenario should use secure broker' */
            )) {
                return (BrowserUtilities.isTeamsHosted() &&
                    (!!new URL(window.location.href).searchParams.get(exports.BROKER_IN_USE_KEY) ||
                        !!((_a = window.sessionStorage) === null || _a === void 0 ? void 0 : _a.getItem(exports.BROKER_IN_USE_KEY))));
            }
            else {
                return (BrowserUtilities.isTeamsBrowserOrDesktop() &&
                    (!!new URL(window.location.href).searchParams.get(exports.BROKER_IN_USE_KEY) ||
                        !!((_b = window.sessionStorage) === null || _b === void 0 ? void 0 : _b.getItem(exports.BROKER_IN_USE_KEY))));
            }
        }
        catch (_c) {
            // no-op, we want to default to not using secure broker
            // (window.location.href is not a valid URL would hit here).
            return false;
        }
    };
    /**
     * Returns true if current page is loaded as iFrame under secure broker app
     * @returns
     */
    BrowserUtilities.isRequestComingFromSecureBroker = function () {
        try {
            if (!SPKillSwitch_1.default.isActivated('6efb55f0-7812-4b62-9443-988f9d8bbdbe'
            /* 02/24/2023, 'Any teams hosted scenario should use secure broker' */
            )) {
                return (BrowserUtilities.isTeamsHosted() &&
                    !!new URL(window.location.href).searchParams.get(exports.BROKER_IN_USE_KEY));
            }
            else {
                return (BrowserUtilities.isTeamsBrowserOrDesktop() &&
                    !!new URL(window.location.href).searchParams.get(exports.BROKER_IN_USE_KEY));
            }
        }
        catch (_a) {
            // no-op, we want to default to not using secure broker
            // (window.location.href is not a valid URL would hit here).
            return false;
        }
    };
    /**
     * Set flags in session storage indicating current page is loaded as iFrame under secure broker app.
     * It is callers responsibility to ensure it is indeed loaded using secure broker app.
     * Also see isUsingSecureBroker()
     */
    BrowserUtilities.setUsingSecureBroker = function () {
        if (window.sessionStorage) {
            try {
                window.sessionStorage.setItem(exports.BROKER_IN_USE_KEY, 'true');
            }
            catch (_a) {
                // We don't want to fail here.  It can prevent link navigation from working
            }
        }
    };
    BrowserUtilities.isInTeamsChannelApp = function () {
        var _a;
        return BrowserUtilities.isTeamsHosted() && BrowserUtilities.safelyAccessLocalStorage(_teamsAppEntityId)
            ? ((_a = BrowserUtilities.safelyAccessLocalStorage(_teamsAppEntityId)) === null || _a === void 0 ? void 0 : _a.indexOf('sharepointtab_')) !== -1
            : false;
    };
    BrowserUtilities.isViva1PHosted = function () {
        try {
            return (BrowserUtilities.isTeamsHosted() &&
                BrowserUtilities.safelyAccessLocalStorage(_teamsAppEntityId) === _vivaConnections1PAppId);
        }
        catch (_a) {
            return false;
        }
    };
    /**
     * Returns true if we are in the VivaTopics App.
     */
    BrowserUtilities.isVivaTopicsHosted = function () {
        try {
            var queryParameters = new URL(window.location.href).searchParams;
            return !!((BrowserUtilities.isTeamsHosted() &&
                BrowserUtilities.safelyAccessLocalStorage(_teamsAppEntityId) === _vivaTopicsAppId) ||
                queryParameters.get('app') === 'topics');
        }
        catch (_a) {
            return false;
        }
    };
    BrowserUtilities.isEduClassHosted = function () {
        try {
            return (BrowserUtilities.isTeamsHosted() &&
                BrowserUtilities.safelyAccessLocalStorage(_teamsAppEntityId) === _classHomeAppId);
        }
        catch (_a) {
            return false;
        }
    };
    BrowserUtilities.isTeamsAndroidHosted = function () {
        if (BrowserUtilities._isTeamsAndroid === undefined) {
            var ua = navigator.userAgent;
            if (SPKillSwitch_1.default.isActivated('ab168592-38f6-4ca4-a5f4-87bc8057da3f'
            /* '03/04/2024', 'Ensure MEE Bridge is considered for Hosted apps' */
            )) {
                BrowserUtilities._isTeamsAndroid =
                    /* Android WebView (scoped to Microsoft Teams for now) */
                    /*
                    Mozilla/5.0 (Linux; Android 9; Mi A1 Build/PKQ1.180917.001; wv) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/81.0.4044.117 Mobile Safari/537.36 TeamsMobile-Android
                    Mozilla/5.0 (Linux; Android 9; Mi A1 Build/PKQ1.180917.001; wv) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/81.0.4044.117 Mobile Safari/537.36 TeamsMobile-Android-dev
                    */
                    /.* TeamsMobile-Android.*/.test(ua);
            }
            else {
                BrowserUtilities._isTeamsAndroid =
                    /* Android WebView (scoped to Microsoft Teams for now) */
                    /*
                    Mozilla/5.0 (Linux; Android 9; Mi A1 Build/PKQ1.180917.001; wv) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/81.0.4044.117 Mobile Safari/537.36 TeamsMobile-Android
                    Mozilla/5.0 (Linux; Android 9; Mi A1 Build/PKQ1.180917.001; wv) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/81.0.4044.117 Mobile Safari/537.36 TeamsMobile-Android-dev
                    Mozilla/5.0 (Linux; Android 9; Mi A1 Build/PKQ1.180917.001; wv) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/81.0.4044.117 Mobile Safari/537.36 Viva Connections for Mobile (MEE App)
                    */
                    /.* (TeamsMobile-Android|Android.*Viva Connections for Mobile).*/.test(ua);
            }
        }
        return BrowserUtilities._isTeamsAndroid;
    };
    BrowserUtilities.isTeamsIOSHosted = function () {
        if (BrowserUtilities._isTeamsIOS === undefined) {
            var ua = navigator.userAgent;
            if (SPKillSwitch_1.default.isActivated('ab168592-38f6-4ca4-a5f4-87bc8057da3f'
            /* '03/04/2024', 'Ensure MEE Bridge is considered for Hosted apps' */
            )) {
                BrowserUtilities._isTeamsIOS =
                    /* Electron WebView (scoped to Microsoft Teams for now) */
                    /*
                    Mozilla/5.0 (iPhone; CPU iPhone OS 12_4 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 TeamsMobile-iOS
                    Mozilla/5.0 (iPhone; CPU iPhone OS 12_4 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 TeamsMobile-iOS-dev
                    */
                    /.* TeamsMobile-iOS.*/.test(ua);
            }
            else {
                BrowserUtilities._isTeamsIOS =
                    /* Electron WebView (scoped to Microsoft Teams for now) */
                    /*
                    Mozilla/5.0 (iPhone; CPU iPhone OS 12_4 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 TeamsMobile-iOS
                    Mozilla/5.0 (iPhone; CPU iPhone OS 12_4 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 TeamsMobile-iOS-dev
                    Mozilla/5.0 (iPhone; CPU iPhone OS 16_6_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Viva Connections for Mobile (MEE App)
                    */
                    /.* (TeamsMobile-iOS|iPhone.*Viva Connections for Mobile).*/.test(ua);
            }
        }
        return BrowserUtilities._isTeamsIOS;
    };
    BrowserUtilities.isTeamsWebViewHostedDeprecated = function () {
        var ua = navigator.userAgent;
        return (
        /* Electron WebView (scoped to Microsoft Teams for now) - Desktop Client scenario */
        /Teams\/((?:(\d+)\.)?(?:(\d+)\.)?(?:(\d+)\.\d+)).* Electron\/((?:(\d+)\.)?(?:(\d+)\.)?(?:(\d+)\.\d+))/.test(ua) ||
            BrowserUtilities.safelyAccessLocalStorage('TeamsHostClientType') ===
                'desktop' /* This will be the case in Teams Desktop app */);
    };
    BrowserUtilities.isTeamsWebViewHosted = function () {
        return (BrowserUtilities.isTeamsHosted() &&
            BrowserUtilities.safelyAccessLocalStorage('TeamsHostClientType') === 'desktop'); /* This will be the case in Teams Desktop app */
    };
    BrowserUtilities.isTeams1PVivaDesktopHosted = function () {
        return (BrowserUtilities.isTeamsWebViewHosted() &&
            BrowserUtilities.safelyAccessLocalStorage(_teamsAppEntityId) === _vivaConnections1PAppId);
    };
    /**
     * When SharePoint is hosted in Teams, it can either be in :
     *    1. Teams Desktop Client: Electron Webview
     *    2. Teams in Browser: Iframe
     * Old Teams:
     * Desktop Client is identifiable via the user-agent.
     * However, for Teams in Browser, since we cannot leverage user-agent,
     * we'll be using window.name property to get the host iframe's name,
     * which when equal to "embedded-page-container" lets us know that
     * SharePoint is currently running in Teams in Browser.
     *
     * For Teams 2.1:
     * window.name property is equal to "embedded-page-container" for desktop
     * and browser. We rely on localStorage from Teams context to find the correct host.
     * If localStorage has not been set in the lifecycle, this api may not return the correct result in new Teams
     */
    BrowserUtilities.isTeamsBrowserHostedDeprecated = function () {
        return window.name === _teamsBrowserWindowName;
    };
    BrowserUtilities.isTeamsBrowserHosted = function () {
        return (BrowserUtilities.isTeamsHosted() &&
            BrowserUtilities.safelyAccessLocalStorage('TeamsHostClientType') === 'web'); /* This will be the case in Teams Browser app */
    };
    BrowserUtilities.isTeamsBrowserOrDesktop = function () {
        return BrowserUtilities.isTeamsWebViewHosted() || BrowserUtilities.isTeamsBrowserHosted();
    };
    /**
     * Returns true, if SharePoint is hosted in Viva Mobile L2 View (browser).
     * If SharePoint is hosted in bridge webview, returns false.
     * If required please add different API for bridge by checking 'VivaBridge' string.
     *
     * In Viva, application name is appended to default RN WebView user agent -
     * application name for L2 View WebView - 'Viva Connections for Mobile (MEE App)'
     * application name for bridge WebView  - 'Viva Connections for Mobile (MEE App) VivaBridge'
     */
    BrowserUtilities.isTeamsVivaMobile = function () {
        if (BrowserUtilities._isTeamsVivaMobile === undefined) {
            BrowserUtilities._isTeamsVivaMobile = false;
            try {
                var ua = navigator.userAgent;
                BrowserUtilities._isTeamsVivaMobile =
                    (/.* Viva Connections for Mobile.*/.test(ua) && ua.indexOf('VivaBridge') === -1) ||
                        /[?&]env=VivaMobile/.test(location.search);
            }
            catch (_) {
                /* no-op */
            }
        }
        return BrowserUtilities._isTeamsVivaMobile;
    };
    BrowserUtilities.isInPortalsApp = function () {
        var queryParameters = new URL(window.location.href).searchParams;
        return ((BrowserUtilities.isTeamsHosted() && queryParameters.get('app') === 'portals') ||
            BrowserUtilities.isViva1PHosted());
    };
    BrowserUtilities.isTemplatesGallery = function () {
        return (window.location.pathname.toLowerCase().indexOf('/_layouts/15/createpagefromtemplate.aspx') !== -1 &&
            (SPExperiment_1.SPExperiment._getVariantWithoutLogging(90090 /* TemplateGalleryV1Experiment */) === 1 ||
                SPFlight_1.default.isEnabled(62046 /* TemplateGallery */)));
    };
    /**
     * Returns true, if Mobile bridge is hosted in mobile Teams browser.
     */
    BrowserUtilities.isMEEBridgeApplicationHosted = function () {
        if (BrowserUtilities._isMEEBridgeApplicationHosted === undefined) {
            BrowserUtilities._isMEEBridgeApplicationHosted = false;
            try {
                BrowserUtilities._isMEEBridgeApplicationHosted =
                    (/.* Viva Connections for Mobile.*/.test(navigator.userAgent) &&
                        navigator.userAgent.indexOf('VivaBridge') !== -1) ||
                        /[?&]env=VivaBridge/.test(location.search);
            }
            catch (e) {
                /* no-op */
            }
        }
        return BrowserUtilities._isMEEBridgeApplicationHosted;
    };
    /**
     * Returns true, if we are in the SharePoint Mobile app.
     */
    BrowserUtilities.isSharePointMobileHosted = function () {
        if (BrowserUtilities._isSharePointMobileHosted === undefined) {
            BrowserUtilities._isSharePointMobileHosted = false;
            try {
                BrowserUtilities._isSharePointMobileHosted = /.*SharePoint for (Android|iOS).*/.test(navigator.userAgent);
            }
            catch (e) {
                /* no-op */
            }
        }
        return BrowserUtilities._isSharePointMobileHosted;
    };
    /**
     * Returns true, if we are in the VCM app.
     */
    BrowserUtilities.isVivaConnectionsMobileHosted = function () {
        if (BrowserUtilities._isVivaConnectionsMobileHosted === undefined) {
            BrowserUtilities._isVivaConnectionsMobileHosted = false;
            // check whether we are in VCM, whether that's via MEE Bridge or a Web View
            try {
                BrowserUtilities._isVivaConnectionsMobileHosted = /.* Viva Connections for Mobile.*/.test(navigator.userAgent);
            }
            catch (e) {
                /* no-op */
            }
        }
        return BrowserUtilities._isVivaConnectionsMobileHosted;
    };
    /**
     * Return true, if SharePoint is hosted in Teams, either Webview or Browser
     */
    BrowserUtilities.isTeamsHostedDeprecated = function () {
        var isWindowNativeInterface = Boolean(window.parent === window.self && window.nativeInterface);
        return (isWindowNativeInterface ||
            BrowserUtilities.isTeamsWebViewHosted() ||
            BrowserUtilities.isTeamsBrowserHosted() ||
            BrowserUtilities.isTeamsAndroidHosted() ||
            BrowserUtilities.isTeamsIOSHosted());
    };
    BrowserUtilities.isTeamsHosted = function () {
        var ua = navigator.userAgent;
        var isElectron = 
        /* Electron WebView (scoped to Microsoft Teams for now) - Desktop Client scenario */
        /Teams\/((?:(\d+)\.)?(?:(\d+)\.)?(?:(\d+)\.\d+)).* Electron\/((?:(\d+)\.)?(?:(\d+)\.)?(?:(\d+)\.\d+))/.test(ua);
        var isEmbeddedContainer = window.name === _teamsBrowserWindowName;
        return (isElectron ||
            isEmbeddedContainer ||
            BrowserUtilities.isTeamsAndroidHosted() ||
            BrowserUtilities.isTeamsIOSHosted());
    };
    BrowserUtilities.isiOSWebView = function () {
        return /(iPhone|iPod|iPad).*AppleWebKit(?!.*Safari)/.test(navigator.userAgent);
    };
    BrowserUtilities.isSharePointiOSApp = function () {
        if (BrowserUtilities._isSharePointiOSApp === undefined) {
            var userAgent = navigator.userAgent;
            BrowserUtilities._isSharePointiOSApp =
                BrowserUtilities.isiOSWebView() || BrowserUtilities._isTestMobileWebView();
            if (this._isMobileIntuneBrowser(userAgent)) {
                BrowserUtilities._isSharePointiOSApp = false;
            }
        }
        return BrowserUtilities._isSharePointiOSApp;
    };
    BrowserUtilities.isMobileWebView = function () {
        if (BrowserUtilities._isMobileWebView === undefined) {
            var ua = navigator.userAgent;
            BrowserUtilities._isMobileWebView =
                /* iOS WebView */
                this.isSharePointiOSApp() ||
                    /* Android WebView (Lollipop+older) */
                    /(Android ).*; wv/.test(ua) ||
                    /(Android ).* Version\/((?:(\d+)\.)?(?:(\d+)\.)?(?:(\d+)\.\d+))/.test(ua) ||
                    /* Windows WebView (C# and JavaScript-based apps) */
                    (navigator.userAgent.indexOf(_sharePointDesktopString) === -1 &&
                        (/(Windows ).*; WebView\/((?:(\d+)\.)?(?:(\d+)\.)?(?:(\d+)\.\d+))/.test(ua) ||
                            /(Windows ).*; MSAppHost\/((?:(\d+)\.)?(?:(\d+)\.)?(?:(\d+)\.\d+))/.test(ua))) ||
                    BrowserUtilities._isTestMobileWebView();
            if (this._isMobileIntuneBrowser(navigator.userAgent)) {
                BrowserUtilities._isMobileWebView = false;
            }
        }
        return BrowserUtilities._isMobileWebView;
    };
    BrowserUtilities.isMobileBrowser = function () {
        if (BrowserUtilities._isMobileBrowser === undefined) {
            //
            // NOTE:
            // Currently matching hosted Android WebKit, iPhone/iPod Mobile Safari and IE Mobile.
            // May be extended to cover other mobile browsers as necessary. Not a perfect science.
            //
            BrowserUtilities._isMobileBrowser =
                BrowserUtilities.isMobileWebView() ||
                    /[?&]env=WebView/.test(location.search) ||
                    /(Android|iPhone|iPod|IEMobile|BlackBerry)/.test(navigator.userAgent) ||
                    /[?&]env=Mobile/.test(location.search);
        }
        return BrowserUtilities._isMobileBrowser;
    };
    BrowserUtilities.isEmbedded = function () {
        if (BrowserUtilities._isEmbedded === undefined) {
            if (BrowserUtilities.isWebViewHosted() || /[?&]PageVersion=[^&]/.test(location.search)) {
                BrowserUtilities._isEmbedded = true;
            }
            else if (navigator.userAgent.indexOf('ManagedBrowser') > -1 ||
                (SPFlight_1.default.isEnabled(60668 /* WexWebPartAudienceTargeting */) &&
                    /[?&]env=Preview/.test(location.search))) {
                BrowserUtilities._isEmbedded = false;
            }
            else {
                try {
                    var isIframeOrPhantomJS = window.parent && window.parent !== window && navigator.userAgent.indexOf('PhantomJS/') === -1;
                    BrowserUtilities._isEmbedded = isIframeOrPhantomJS || /[?&]env=Embedded/.test(location.search);
                }
                catch (e) {
                    BrowserUtilities._handleErrorForParentWindowCheck();
                }
            }
        }
        return !!BrowserUtilities._isEmbedded;
    };
    /**
     * Returns true if SharePoint site is embedded within site and not hosted e.g. in Teams.
     */
    BrowserUtilities.isEmbeddedInSite = function () {
        if (BrowserUtilities._isEmbeddedInSite === undefined) {
            BrowserUtilities._isEmbeddedInSite =
                (BrowserUtilities.isEmbedded() && !BrowserUtilities.isTeamsHosted()) ||
                    BrowserUtilities._isEmbeddedInTeamsClient();
        }
        return BrowserUtilities._isEmbeddedInSite;
    };
    BrowserUtilities.supportsServiceWorker = function () {
        return _serviceWorkerProperty in navigator;
    };
    BrowserUtilities.supportsNavigationPreload = function () {
        // ServiceWorkerRegistration is available on global window object
        return (!!window.ServiceWorkerRegistration &&
            _navigationPreloadProperty in window.ServiceWorkerRegistration.prototype);
    };
    BrowserUtilities.isOutlookWebViewHosted = function () {
        if (SPKillSwitch_1.default.isActivated('05dfabf6-5844-4295-b4f1-07adfe1700d0'
        /* '03/02/2022', 'Detect SharePoint page hosted in Outlook WebView' */
        )) {
            return false;
        }
        try {
            return /[?&]env=OutlookWebView/.test(location.search);
        }
        catch (e) {
            return false;
        }
    };
    /**
     * Check if the current browser is Safari. We have this check because
     * cookie based scenarios do not work in Safari due to ITP.
     *
     * @returns true if the current browser is Safari.
     */
    BrowserUtilities.isSafari = function () {
        if (this._isSafari === undefined) {
            // Check for 'Chrome' since Chromium browsers add 'Safari' to their user agent string
            this._isSafari =
                navigator.userAgent.indexOf('Chrome') === -1 && navigator.userAgent.indexOf('Safari') > -1;
        }
        return this._isSafari;
    };
    /**
     * Returns true if we are on the VivaHome layouts page.
     */
    BrowserUtilities.isVivaHome = function () {
        return (SPFlight_1.default.isEnabled(60840 /* VivaHome */) &&
            window.location.pathname.toLowerCase().indexOf('/_layouts/15/vivahome.aspx') !== -1);
    };
    /**
     * Returns true if user is navigated from the VivaHome layouts page.
     */
    BrowserUtilities.isFromVivaHome = function () {
        return (!SPKillSwitch_1.default.isActivated('e67cbc4f-48e9-4078-9352-269082e1631c'
        /* '10/20/2022', 'if user is navigated from the VivaHome layouts page' */
        ) && window.location.search.indexOf('isFromVivaHome=true') !== -1);
    };
    /**
     * Returns true if the workbench page is opened.
     */
    BrowserUtilities.isWorkbenchHosted = function () {
        return location.href.indexOf('_layouts/15/workbench.aspx') !== -1;
    };
    /**
     * A call to access localStorage will throw an error if called by a cross-domain iframe,
     * if the call localStorage throws an error, catch the error and return null instead
     *
     * @returns the item stored in localStorage at the given key or null if the item is not found.
     */
    // eslint-disable-next-line @rushstack/no-new-null
    BrowserUtilities.safelyAccessLocalStorage = function (key) {
        try {
            return localStorage === null || localStorage === void 0 ? void 0 : localStorage.getItem(key);
        }
        catch (error) {
            if (!SPKillSwitch_1.default.isActivated('274b24bf-e198-427e-ade6-a472ecbb02c3'
            /* '05/18/2023', 'Do not throw if fail to access localStorage' */
            )) {
                return null;
            }
            else {
                throw error;
            }
        }
    };
    /**
     * A call to access sessionStorage will throw an error if called by a cross-domain iframe,
     * if the call sessionStorage throws an error, catch the error and return null instead
     *
     * @returns the item stored in sessionStorage at the given key or null if the item is not found.
     */
    // eslint-disable-next-line @rushstack/no-new-null
    BrowserUtilities.safelyAccessSessionStorage = function (key) {
        try {
            return sessionStorage === null || sessionStorage === void 0 ? void 0 : sessionStorage.getItem(key);
        }
        catch (error) {
            if (!SPKillSwitch_1.default.isActivated('274b24bf-e198-427e-ade6-a472ecbb02c3'
            /* '05/18/2023', 'Do not throw if fail to access sessionStorage' */
            )) {
                return null;
            }
            else {
                throw error;
            }
        }
    };
    /**
     * Returns true if SharePoint site is embedded within SharePoint which is hosted in Teams client.
     * This check is not required for Teams browser because, this api needs to be used along with isTeamsHosted,
     * which will return false for browser anyway
     */
    BrowserUtilities._isEmbeddedInTeamsClient = function () {
        if (BrowserUtilities.__isEmbeddedInTeamsClient === undefined) {
            BrowserUtilities.__isEmbeddedInTeamsClient = false;
            if (BrowserUtilities.isTeamsWebViewHosted()) {
                try {
                    BrowserUtilities.__isEmbeddedInTeamsClient = window.name !== _teamsClientWindowName;
                }
                catch (e) {
                    /* no-op */
                }
            }
        }
        return BrowserUtilities.__isEmbeddedInTeamsClient;
    };
    BrowserUtilities._isTestMobileWebView = function () {
        return /[?&]env=MobileWebView/.test(location.search);
    };
    BrowserUtilities._isMobileIntuneBrowser = function (userAgent) {
        return navigator.userAgent.indexOf('ManagedBrowser') > -1;
    };
    BrowserUtilities._handleErrorForParentWindowCheck = function () {
        //
        // Accessing window.parent x-domain throws - then window.parent !== window,
        // and we can assume we are hosted in an x-domain iframe
        //
        BrowserUtilities._isEmbedded = true;
    };
    return BrowserUtilities;
}());
exports.BrowserUtilities = BrowserUtilities;
//# sourceMappingURL=BrowserUtilities.js.map