"use strict";
// Copyright (c) Microsoft. All rights reserved.
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Indicates whether a visual control should display itself for reading or for editing.
 *
 * @public
 */
var DisplayMode;
(function (DisplayMode) {
    /**
     * The page, control, or editing surface is in a mode intended for reading.  It may have some interactive
     * features, but the authoring controls are not enabled.
     */
    DisplayMode[DisplayMode["Read"] = 1] = "Read";
    /**
     * The page, control, or editing surface is in a mode intended for authoring new content.  It may display
     * editing panels or other controls that are not part of the normal reading experience.
     */
    DisplayMode[DisplayMode["Edit"] = 2] = "Edit";
    /**
     * The page, control, or editing surface is in a mode intended for preview the saved content. It could
     * preview the content in different devices and targeting audiences.
     *
     * @alpha
     */
    DisplayMode[DisplayMode["Preview"] = 3] = "Preview";
})(DisplayMode || (DisplayMode = {}));
exports.default = DisplayMode;
//# sourceMappingURL=DisplayMode.js.map