"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EnvironmentType = void 0;
var tslib_1 = require("tslib");
var Validate_1 = tslib_1.__importDefault(require("./Validate"));
var SPCoreLibraryStrings_resx_1 = tslib_1.__importDefault(require("./SPCoreLibraryStrings.resx"));
/**
 * Indicates the general type of environment where the SharePoint Framework is running.
 *
 * @remarks
 * This enum is used by the {@link Environment."type"} property.  It provides a way to distinguish
 * environments where certain functionality may be limited or disabled.
 *
 * @public
 */
var EnvironmentType;
(function (EnvironmentType) {
    /**
     * Indicates that the SharePoint Framework is running inside a test harness, e.g. a unit test.
     * There may be no user interaction at all, and network access in general may be disabled.
     */
    EnvironmentType[EnvironmentType["Test"] = 0] = "Test";
    /**
     * Indicates that the SharePoint Framework is running on a page from a "localhost" web server,
     * for example the SharePoint Workbench when hosted via "gulp serve".  SharePoint REST API calls
     * will not be available, and in general network access may not authenticate properly.
     * Certain page context information may be using mock values for testing.
     *
     * @deprecated The local workbench has been removed. Use {@link EnvironmentType.SharePoint} instead.
     */
    EnvironmentType[EnvironmentType["Local"] = 1] = "Local";
    /**
     * Indicates that the SharePoint Framework is running on a modern SharePoint web page,
     * with full framework functionality.  This is the normal usage scenario.
     */
    EnvironmentType[EnvironmentType["SharePoint"] = 2] = "SharePoint";
    /**
     * Indicates that the framework was hosted by a classic server-rendered SharePoint page.
     * Some functionality may be limited, e.g. various extensibility APIs may not be supported.
     */
    EnvironmentType[EnvironmentType["ClassicSharePoint"] = 3] = "ClassicSharePoint";
})(EnvironmentType || (exports.EnvironmentType = EnvironmentType = {}));
/**
 * Information about the system environment where the SharePoint Framework is running.
 * @public
 */
var Environment = /** @class */ (function () {
    function Environment() {
    }
    /**
     * This is called once by the system during startup to initialize the Environment object.
     * @internal
     */
    Environment._initialize = function (data) {
        Validate_1.default.isNotNullOrUndefined(data, 'data');
        Environment._type = data.type;
        Validate_1.default.isNotNullOrUndefined(Environment._type, 'environmentType');
        if (!EnvironmentType.hasOwnProperty(data.type.toString())) {
            throw new Error(SPCoreLibraryStrings_resx_1.default.invalidEnvironmentTypeError);
        }
    };
    Object.defineProperty(Environment, "type", {
        /**
         * Indicates the general type of environment where the SharePoint Framework is running.
         */
        get: function () {
            return this._type;
        },
        enumerable: false,
        configurable: true
    });
    return Environment;
}());
exports.default = Environment;
//# sourceMappingURL=Environment.js.map