"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.safeWindowOpen = void 0;
/**
 * `window.open` can execute arbitrary JavaScript if passed an argument with the `javascript` protocol.
 * This function validates the protocol before invoking `window.open`.
 *
 * @remarks This function should be used over `window.open`.
 *
 * @param url - Target url to open.
 *
 * @internal
 */
function safeWindowOpen(url) {
    var targetUrl = new URL(url);
    // eslint-disable-next-line no-script-url
    if (targetUrl.protocol === 'javascript:') {
        throw new Error('javascript protocol is disallowed');
    }
    window.open(targetUrl);
}
exports.safeWindowOpen = safeWindowOpen;
//# sourceMappingURL=SafeWindowOpen.js.map