"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var MULTIPLIER = 48271;
var MODULUS = 2147483647;
var MODULUS_HIGH = Math.floor(MODULUS / MULTIPLIER);
var MODULUS_LOW = MODULUS % MULTIPLIER;
/**
 * This is a test implementation of IRandomNumberGenerator that provides
 * a reproducible sequence of reasonably pseudorandom numbers.
 */
var MockRandomNumberGenerator = /** @class */ (function () {
    function MockRandomNumberGenerator(serviceScope) {
        this.reset();
    }
    /**
     * Reset the sequence.
     */
    MockRandomNumberGenerator.prototype.reset = function (seed) {
        // The seed cannot be zero
        this._seed = seed && seed !== 0 ? seed : 1;
    };
    /**
     * IRandomNumberGenerator implementation
     */
    MockRandomNumberGenerator.prototype.generate = function () {
        return this._generateInt32() / MODULUS;
    };
    MockRandomNumberGenerator.prototype._generateInt32 = function () {
        // This is the classic Lehmer random number generator.  (I love codin')
        var seedHi = Math.floor(this._seed / MODULUS_HIGH);
        var seedLow = this._seed % MODULUS_HIGH;
        this._seed = MULTIPLIER * seedLow - MODULUS_LOW * seedHi;
        if (this._seed <= 0) {
            this._seed += MODULUS;
        }
        return this._seed;
    };
    return MockRandomNumberGenerator;
}());
exports.default = MockRandomNumberGenerator;
//# sourceMappingURL=MockRandomNumberGenerator.js.map